<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***/
class BOOKMARKS_CTRL_List extends OW_ActionController
{
    private $service;
    
    public function __construct()
    {
        parent::__construct();
        
        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new AuthenticateException();
        }
        
        $sortCtrl = new BASE_CMP_SortControl();
        $sortCtrl->addItem(BOOKMARKS_BOL_Service::LIST_LATEST, OW::getLanguage()->text('bookmarks', 'latest'), OW::getRouter()->urlForRoute('bookmarks.list', array('category' => BOOKMARKS_BOL_Service::LIST_LATEST)));
        $sortCtrl->addItem(BOOKMARKS_BOL_Service::LIST_ONLINE, OW::getLanguage()->text('bookmarks', 'online'), OW::getRouter()->urlForRoute('bookmarks.list', array('category' => BOOKMARKS_BOL_Service::LIST_ONLINE)));
        $this->addComponent('sort', $sortCtrl);
        
        $this->service = BOOKMARKS_BOL_Service::getInstance();
    }
    
    public function init() 
    {
        parent::init();
        
        $handler = OW::getRequestHandler()->getHandlerAttributes();
        
        if ( !in_array($handler[OW_RequestHandler::ATTRS_KEY_VARLIST]['category'], 
            array(BOOKMARKS_BOL_Service::LIST_LATEST, BOOKMARKS_BOL_Service::LIST_ONLINE)) )
        {
            throw new Redirect404Exception();
        }
        
        $this->getComponent('sort')->setActive($handler[OW_RequestHandler::ATTRS_KEY_VARLIST]['category']);
    }

    public function getList( array $params )
    {
        OW::getDocument()->setHeading(OW::getLanguage()->text('bookmarks', 'list_headint_title'));
        $this->setTemplate(OW::getPluginManager()->getPlugin('bookmarks')->getCtrlViewDir() . 'list.html');
        
        $userId = OW::getUser()->getId();
        $page = (!empty($_GET['page']) && intval($_GET['page']) > 0 ) ? $_GET['page'] : 1;
        $userOnPage = (int)OW::getConfig()->getValue('base', 'users_on_page');
        $first = ($page - 1) * $userOnPage;
        
        $list = $this->service->findBookmarksUserIdList($userId, $first, $userOnPage, $params['category']);
        $count = $this->service->findBookmarksCount($userId, $params['category']);

        $sexValue = array();
        $userDataList = array();
        $questionService = BOL_QuestionService::getInstance();
        $data = $questionService->getQuestionData($list, array('sex', 'googlemap_location', 'birthdate'));

        foreach ( BOL_QuestionValueDao::getInstance()->findQuestionValues('sex') as $sexDto )
        {
            $sexValue[$sexDto->value] = $questionService->getQuestionValueLang('sex', $sexDto->value);
        }
        
        foreach ( $data as $userId => $user )
        {
            if ( isset($user['birthdate']) )
            {
                $date = UTIL_DateTime::parseDate($user['birthdate'], UTIL_DateTime::MYSQL_DATETIME_DATE_FORMAT);
                $age = UTIL_DateTime::getAge($date['year'], $date['month'], $date['day']);
            }
            else
            {
                $age = '';
            }
            
            $userDataList[$userId] = array(
                'info_gender' => (!empty($user['sex']) && !empty($sexValue[$user['sex']])) ? $sexValue[$user['sex']] : '' . ' ' . $age,
                'location' => !empty($user['googlemap_location']) ? $user['googlemap_location']['address'] : ''
            );
        }
        
        $this->addComponent('list', OW::getClassInstance('BASE_CMP_Users', $userDataList, array(), $count));
    }
}
