<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class HOTLIST_BOL_Service {

    /**
     *
     * @var HOTLIST_BOL_UserDao
     */
    private $userDao;
    /**
     * Class instance
     *
     * @var HOTLIST_BOL_Service
     */
    private static $classInstance;

    /**
     * Class constructor
     *
     */
    protected function __construct() {
        $this->userDao = HOTLIST_BOL_UserDao::getInstance();
    }

    /**
     * Returns class instance
     *
     * @return HOTLIST_BOL_Service
     */
    public static function getInstance() {
        if (self::$classInstance === null) {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Add a message entries to a database.
     */
    public function addUser($userId)
    {
        $user = new HOTLIST_BOL_User();
        $user->userId = $userId;
        $user->timestamp = time();
        $user->expiration_timestamp = time() + OW::getConfig()->getValue('hotlist', 'expiration_time');

        $this->userDao->save($user);
    }

    public function deleteUser($userId)
    {
        return $this->userDao->deleteByUserId($userId);
    }

    public function getUserCount()
    {
        return $this->userDao->countAll();
    }

    public function clearExpiredUsers()
    {
        $userList = $this->userDao->findExpiredUsers();
        
        if (empty($userList))
        {
            return;
        }
        
//        foreach($userList as $user)
//        {
//            //Newsfeed
//            OW::getEventManager()->trigger(new OW_Event('feed.delete_item', array(
//                'entityType' => 'add_to_hotlist',
//                'entityId' => $user->userId
//            )));
//        }
        $this->userDao->clearExpiredUsers();
    }

    public function getHotList( $start = 0, $count = null, $excludeList = array() )
    {
        return $this->userDao->findHotList($start, $count, $excludeList);
    }

    public function findUserById($userId)
    {
        return $this->userDao->findUserById($userId);
    }
}
