<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class MEMBERSHIP_CLASS_MembershipPlanProductAdapter implements OW_BillingProductAdapter
{
    const PRODUCT_KEY = 'membership_plan';

    const RETURN_ROUTE = 'membership_subscribe';

    public function getProductKey()
    {
        return self::PRODUCT_KEY;
    }

    public function getProductOrderUrl()
    {
        return OW::getRouter()->urlForRoute(self::RETURN_ROUTE);
    }

    public function deliverSale( BOL_BillingSale $sale )
    {
        $planId = $sale->entityId;

        $membershipService = MEMBERSHIP_BOL_MembershipService::getInstance();

        $plan = $membershipService->findPlanById($planId);
        $type = $membershipService->findTypeByPlanId($planId);

        if ( $plan && $type )
        {
            $isRebill = false;
            if ( $sale->recurring )
            {
                /* @var $currentMembership MEMBERSHIP_BOL_MembershipUser */
                $currentMembership = $membershipService->getUserMembership($sale->userId);

                if ( $currentMembership && $currentMembership->recurring && $currentMembership->typeId == $type->id )
                {
                    $isRebill = true;
                }
            }

            $userMembership = new MEMBERSHIP_BOL_MembershipUser();
            
            $userMembership->userId = $sale->userId;
            $userMembership->typeId = $type->id;
            $userMembership->expirationStamp = time() + (int) $plan->period * $membershipService->getInstance()->getPeriodUnitFactor($plan->periodUnits);
            $userMembership->recurring = $sale->recurring;
    
            $membershipService->setUserMembership($userMembership);

            $title = $membershipService->getMembershipTitle($type->roleId);

            if ( $isRebill )
            {
                $membershipService->sendMembershipRenewedNotification($sale->userId, $title);
            }
            else
            {
                $membershipService->sendMembershipPurchasedNotification($sale->userId, $title, $userMembership->expirationStamp);
            }
    
            return true;
        }
        
        return false;
    }
}