<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class OXDATE_CMP_MobileExperience extends BASE_CLASS_Widget
{
    const IOS = 'oxdateios';
    const ANDROID = 'oxandroid';

    public function __construct( BASE_CLASS_WidgetParameter $paramObj )
    {
        parent::__construct();

        $active = array(
            self::IOS => OW::getPluginManager()->isPluginActive(self::IOS),
            self::ANDROID => OW::getPluginManager()->isPluginActive(self::ANDROID)
        );

        if ( !$active[self::IOS] && !$active[self::ANDROID] )
        {
            $this->setVisible(false);

            return;
        }
        elseif ( empty($paramObj->customParamList['banners']) )
        {
            $this->setVisible(false);

            return;
        }

        $banners = $paramObj->customParamList['banners'];
        $banners = array_merge(array($paramObj->customParamList['show_first'] => true), $banners);

        $event = new BASE_CLASS_EventCollector('app.promo_info');
        OW::getEventManager()->trigger($event);
        $data = call_user_func_array('array_merge', $event->getData());
        $promos = array();

        foreach ( array_keys($banners) as $bunner )
        {
            if ( !isset($data[$bunner]) )
            {
                continue;
            }

            $promos[$bunner] = array(
                'app_url' => $data[$bunner]['app_url']
            );
        }

        if ( count($promos) === 0 )
        {
            $this->setVisible(false);

            return;
        }

        OW::getDocument()->addStyleSheet(OW::getPluginManager()->getPlugin('oxdate')->getStaticCssUrl() . 'mobile_experience.css');
        $this->assign('promos', $promos);
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_TITLE => OW::getLanguage()->text('oxdate', 'mobile_experience_widget_title'),
            self::SETTING_ICON => self::ICON_PICTURE,
            self::SETTING_SHOW_TITLE => false,
            self::SETTING_WRAP_IN_BOX => false
        );
    }

    public static function getSettingList()
    {
        $language = OW::getLanguage();
        $pluginManager = OW::getPluginManager();

        return array(
            'banners' => array(
                'presentation' => self::PRESENTATION_CUSTOM,
                'label' => $language->text('oxdate', 'banners_label'),
                'render' => function( $uniqName, $name, $value )
                {
                    OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('oxdate')->getStaticJsUrl() . 'mobile_experience.js');
                    OW::getDocument()->addOnloadScript(UTIL_JsGenerator::composeJsString(';window.OXDATE_ME_SETTINGS({$params});', array(
                        'params' => array(
                            'iosActive' => OW::getPluginManager()->isPluginActive(OXDATE_CMP_MobileExperience::IOS),
                            'androidActive' => OW::getPluginManager()->isPluginActive(OXDATE_CMP_MobileExperience::ANDROID)
                        )
                    )));

                    $input = new CheckboxGroup('banners');
                    $input->setOptions(array(
                        OXDATE_CMP_MobileExperience::IOS => OW::getLanguage()->text('oxdate', 'ios_label'),
                        OXDATE_CMP_MobileExperience::ANDROID => OW::getLanguage()->text('oxdate', 'android_label')
                    ));
                    $input->setColumnCount(2);
                    $input->setValue(!empty($value) ? array_keys($value) : null);

                    return $input->renderInput();
                },
                'value' => array(
                    self::IOS => $pluginManager->isPluginActive(self::IOS),
                    self::ANDROID => $pluginManager->isPluginActive(self::ANDROID)
                )
            ),
            'show_first' => array(
                'presentation' => self::PRESENTATION_CUSTOM,
                'label' => $language->text('oxdate', 'show_first_label'),
                'render' => function( $uniqName, $name, $value )
                {
                    $input = new RadioField('show_first[]');
                    $input->setOptions(array(
                        OXDATE_CMP_MobileExperience::IOS => OW::getLanguage()->text('oxdate', 'ios_label'),
                        OXDATE_CMP_MobileExperience::ANDROID => OW::getLanguage()->text('oxdate', 'android_label')
                    ));
                    $input->setValue($value);
                    $input->setColumnCount(2);

                    return $input->renderInput();
                },
                'value' => $pluginManager->isPluginActive(self::IOS) ? self::IOS : self::ANDROID
            )
        );
    }

    public static function processSettingList( $settingList, $place, $isAdmin )
    {
        $settingList = parent::processSettingList($settingList, $place, $isAdmin);

        $settingList['show_first'] = array_shift($settingList['show_first']);
        $settingList['banners'] = array_flip($settingList['banners']);

        if ( !OW::getPluginManager()->isPluginActive(self::IOS) && isset($settingList['banners'][self::IOS]) )
        {
            unset($settingList['banners'][self::IOS]);
        }

        if ( !OW::getPluginManager()->isPluginActive(self::ANDROID) && isset($settingList['banners'][self::ANDROID]) )
        {
            unset($settingList['banners'][self::ANDROID]);
        }

        return $settingList;
    }
}