<?php

/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/

class PCGALLERY_CMP_GallerySettings extends OW_Component
{
    public function __construct( $userId ) 
    {
        parent::__construct();
        
        $data = OW::getEventManager()->call("photo.entity_albums_find", array(
            "entityType" => "user",
            "entityId" => $userId
        ));
        
        $albums = empty($data["albums"]) ? array() : $data["albums"];
        
        $source = BOL_PreferenceService::getInstance()->getPreferenceValue("pcgallery_source", $userId);
        $this->assign("source", $source == "album" ? "album": "all");
        
        $selectedAlbum = BOL_PreferenceService::getInstance()->getPreferenceValue("pcgallery_album", $userId);
        
        $form = new Form("pcGallerySettings");
        $form->setEmptyElementsErrorMessage(null);
        $form->setAction(OW::getRouter()->urlFor("PCGALLERY_CTRL_Gallery", "saveSettings"));
        
        $element = new HiddenField("userId");
        $element->setValue($userId);
        $form->addElement($element);
        
        $element = new Selectbox("album");
        $element->setHasInvitation(true);
        $element->setInvitation(OW::getLanguage()->text("pcgallery", "settings_album_invitation"));
        
        $validator = new PCGALLERY_AlbumValidator();
        $element->addValidator($validator);
        
        $albumsPhotoCount = array();
        
        foreach ( $albums as $album ) 
        {
            $element->addOption($album["id"], $album["name"] . " ({$album["photoCount"]})");
            $albumsPhotoCount[$album["id"]] = $album["photoCount"];
            
            if ( $album["id"] == $selectedAlbum )
            {
                $element->setValue($album["id"]);
            }
        }
        
        OW::getDocument()->addOnloadScript(UTIL_JsGenerator::composeJsString('window.pcgallery_settingsAlbumCounts = {$albumsCount};', array(
            "albumsCount" => $albumsPhotoCount
        )));
        
        $element->setLabel(OW::getLanguage()->text("pcgallery", "source_album_label"));
        
        $form->addElement($element);
        
        $submit = new Submit("save");
        $submit->setValue(OW::getLanguage()->text("pcgallery", "save_settings_btn_label"));
        $form->addElement($submit);
        
        $this->addForm($form);
    }
}

class PCGALLERY_AlbumValidator extends OW_Validator
{
    public function getError() {
        return OW::getLanguage()->text("pcgallery", "settings_album_required");
    }

    public function isValid($value) {
        return true;
    }
    
    public function getJsValidator() {
        return "{
            validate : function( value ){
                if ( $('#ugallery-source-album').get(0).checked ) {
                    if ( !value ) throw " . json_encode($this->getError()) . ";
                    var photoCount = parseInt(window.pcgallery_settingsAlbumCounts[value]);
                    
                    if ( photoCount < 10 ) {
                        throw " . json_encode(OW::getLanguage()->text("pcgallery", "settings_album_not_enough_photos")) . ";
                    }
                }
            }
        }";
    }
}