<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class PROTECTEDPHOTOS_CLASS_FriendListFormElement extends PROTECTEDPHOTOS_CLASS_FormElement
{
    public $friendList = array();
    public $selectedList = array();
    public $input;

    public function __construct( $name, $userId )
    {
        parent::__construct($name);

        $this->input = new HiddenField($this->getName());

        $count = OW::getEventManager()->call('plugin.friends.count_friends', array(
            'userId' => $userId
        ));

        if ( (int) $count > 0 )
        {
            $friendIds = OW::getEventManager()->call('plugin.friends.get_friend_list', array(
                'userId' => $userId,
                'count' => $count
            ));
            $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($friendIds);
            $avatars = BOL_AvatarService::getInstance()->getAvatarsUrlList($friendIds);

            foreach ( $friendIds as $friendId )
            {
                $this->friendList[$friendId] = array(
                    'id' => $friendId,
                    'displayName' => $displayNames[$friendId],
                    'src' => $avatars[$friendId]
                );
            }
        }
    }

    public function setSelectedList( array $selectedList )
    {
        $this->selectedList = $selectedList;
    }

    public function setValue( $value )
    {
        $this->input->setValue($value);
    }

    public function getValue()
    {
        return $this->input->getValue();
    }

    public function getElementJs()
    {
        return $this->input->getElementJs();
    }

    public function renderInput( $params = null )
    {
        parent::renderInput($params);

        $markup = '{$input}
        <div class="ow_pass_protected_userlist ow_border">
            <div class="ow_privacy_userlist_main">
                {$friends}
            </div>
        </div>';

        $objRef = $this;
        
        $list = array_reduce($this->friendList, function( $carry, $friend ) use ($objRef)
        {
            $carry .= $objRef->replace('<div class="ow_privacy_userlist_main_item ow_border clearfix" data-id="{$id}" data-display-name="{$displayName}" data-src="{$src}">
                <input type="checkbox" class="ow_privacy_userlist_checkbox" {$checked}>
                <div class="ow_privacy_userlist_item_avatar">
                    <div class="ow_avatar">
                        <img style="max-width: 100%;" title="{$displayName}" alt="{$displayName}" src="{$src}">
                    </div>
                </div>
                <div class="ow_privacy_userlist_item_info">
                    <div class="ow_privacy_userlist_item_info_string ow_small">
                        <b>{$displayName}</b>
                    </div>
                </div>
            </div>', array_merge(
                array(
                    'checked' => (in_array($friend['id'], $objRef->selectedList) ? 'checked="checked"' : '')
                ),
                $friend
            ));

            return $carry;
        }, '');

        return $this->replace($markup, array(
            'input' => $this->input->renderInput(),
            'friends' => $list
        ));
    }
}
