<?php
/***
 * This software is intended for use with Oxdate Free Dating Software
 * http://www.oxdate.org/ and is licensed under The BSD license.
 * Full text of this license can be found at http://www.oxdate.org/license

 * =============================================================================
 * Copyright (c) 2016 Oxdate.org - All rights reserved.
 * =============================================================================

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * Neither the name of the Oxdate Foundation nor the names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***/
class PROTECTEDPHOTOS_MCLASS_EventHandler
{
    private static $instance;

    public static function getInstance()
    {
        if ( null === self::$instance )
        {
            self::$instance = new self();
        }

        return self::$instance;
    }

    private $service;

    private function __construct()
    {
        $this->service = PROTECTEDPHOTOS_BOL_Service::getInstance();
    }

    public function init()
    {
        $eventManager = OW::getEventManager();

        $eventManager->bind('photo.getPhotoList', array($this, 'handlerQueryGetPhotoList'));
        $eventManager->bind('feed.on_item_render', array($this, 'onFeedItemRender'));

        PROTECTEDPHOTOS_CLASS_EventHandler::getInstance()->genericInit();
    }

    public function handlerQueryGetPhotoList( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        if ( OW::getUser()->isAuthorized('photo') ) return;

        $params = $builderEvent->getParams();

        switch ( true )
        {
            case in_array($params['listType'], array(
                'countAlbums',
                'getUserAlbumList',
                'latest',
                'toprated'
            ), true):
                $this->handlerQueryForAlbum($builderEvent);
                break;
            case in_array($params['listType'], array(
                'getAlbumPhotos',
                'countAlbumPhotos',
                'countPhotos',
                'findPhotoListByUserId',
                'getAlbumAllPhotos',
                'findEntityPhotoList'
            ), true):
                $this->handlerQueryForPhoto($builderEvent);
                break;
        }
    }

    private function handlerQueryForAlbum( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();
        $aliases = $params['aliases'];

        $sql = 'LEFT JOIN `%s` AS `pwd` ON(`%s`.`id` = `pwd`.`albumId`)';
        $builderEvent->addJoin(sprintf($sql, PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(), $aliases['album']));
        $builderEvent->addWhere('`pwd`.`albumId` IS NULL');
    }

    private function handlerQueryForPhoto( BASE_CLASS_QueryBuilderEvent $builderEvent )
    {
        $params = $builderEvent->getParams();
        $aliases = $params['aliases'];

        $sql = 'LEFT JOIN `%s` AS `pwd` ON(`%s`.`albumId` = `pwd`.`albumId`)';
        $builderEvent->addJoin(sprintf($sql, PROTECTEDPHOTOS_BOL_PasswordDao::getInstance()->getTableName(), $aliases['photo']));
        $builderEvent->addWhere('`pwd`.`albumId` IS NULL');
    }

    public function onFeedItemRender( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !in_array($params['action']['entityType'], array('photo_comments', 'multiple_photo_upload'), true) )
        {
            return;
        }

        $data = $event->getData();
        $photoId = !empty($data['photoIdList']) ? $data['photoIdList'][0] : $params['action']['entityId'];
        $photoEvent = OW::getEventManager()->trigger(
            new OW_Event('photo.find', array('photoId' => $photoId))
        );
        $photoEventData = $photoEvent->getData();

        if ( !isset($photoEventData['photo']) )
        {
            return;
        }

        $photo = $photoEventData['photo'];
        $plugin = OW::getPluginManager()->getPlugin('protectedphotos');
        $photoUrl = $plugin->getStaticUrl() . 'images/ic_pass_protected.svg';
        $content = $data['content'];

        switch ( $params['action']['entityType'] )
        {
            case 'photo_comments':
                if ( $this->service->isAlbumProtected($photo['albumId']) )
                {
                    $content['vars']['image'] = $photoUrl;
                    $content['vars']['class'] = 'owm_protected_photo_widget_icon';
                    OW::getDocument()->addStyleSheet($plugin->getStaticCssUrl() . 'mobile.css');
                    $data['disabled'] = true;
                }
                break;
            case 'multiple_photo_upload':
                if ( $this->service->isAlbumProtected($photo['albumId']) )
                {
                    foreach ( $content['vars']['list'] as $index => $info )
                    {
                        $content['vars']['list'][$index]['image'] = $photoUrl;
                        $content['vars']['list'][$index]['class'] = 'owm_protected_photo_widget_icon';
                    }

                    OW::getDocument()->addStyleSheet($plugin->getStaticCssUrl() . 'mobile.css');
                    $data['disabled'] = true;
                }
                break;
        }

        $data['content'] = $content;

        $event->setData($data);

    }
}
